/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-core library is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 digitalSTROM ds485-core library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM ds485-core library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * @file
 * Queue storage and initialization of the qp-framework.
 */
#include "qp_port.h"
#include "receiver.h"
#include "mmi.h"
#include "digitalSTROM/ds485.h"
#ifdef DS485_API_V1
#include "ds485-qp.v1.h"
#include "digitalSTROM/ds485-const.v1.h"
#else
#include "ds485-qp.h"
#include "digitalSTROM/ds485-const.h"
#endif
#include "ds485-signals.h"
#include "ds485-proto.h"

static QSubscrList l_subscrSto[MAX_PUBLISH_SIG];
static QEvent const * l_receiverQueueSto[1024];   /* event queue storage */
static QEvent const * l_ds485QueueSto[64];      /* event queue storage */
static QEvent const * l_dsmApiQueueSto[16];     /* event queue storage */
static QEvent const * l_mmiQueueSto[70];     /* event queue storage */

typedef union {
    OtherByteEvt otherByte;
    DS485MessageEvt ds485;
} SmallPoolUnion;

typedef union {
    DS485LongMessageEvt ds485;
    MMI_DS485DsidRegisteredEvt dsid;
    DS485PacketEvt  packet;
} MediumPoolUnion;

typedef union {
    DS485_DsidResponseEvt    dsid;
} LargePoolUnion;

static SmallPoolUnion l_smallPoolSto[512];
static MediumPoolUnion l_mediumPoolSto[256];
static LargePoolUnion l_largePoolSto[2];

void libqp_init(void)
{
    uint8_t prio;

    BSP_init();

    /* initialize the framework and the underlying RT kernel */
    QF_init();

    /* initialize publish-subscribe */
    QF_psInit(l_subscrSto, Q_DIM(l_subscrSto));

    /* instantiate active objects */
    Receiver_ctor();
    DS485_ctor();
    DsmApi_ctor();
    MMI_ctor();

    /* initialize the event pools */
    QF_poolInit(l_smallPoolSto, sizeof(l_smallPoolSto), sizeof(l_smallPoolSto[0]));
    QF_poolInit(l_mediumPoolSto, sizeof(l_mediumPoolSto), sizeof(l_mediumPoolSto[0]));
    QF_poolInit(l_largePoolSto, sizeof(l_largePoolSto), sizeof(l_largePoolSto[0]));

    /* top state function dictionary entry */
    QS_FUN_DICTIONARY(&QHsm_top);

    /* published signals dictionary entries */
    QS_SIG_DICTIONARY(START_SIG, 0);
    QS_SIG_DICTIONARY(CONFIG_INIT_SIG, 0);
    QS_SIG_DICTIONARY(DS485_ADDRESS_SIG, 0);

    /* start the active objects */
    prio = 0;

    QActive_start(AO_MMI,                                           /* global pointer to the active object */
                  ++prio,                                           /* priority */
                  l_mmiQueueSto, Q_DIM(l_mmiQueueSto),              /* event queue */
                  (void*)0, 0,                                      /* no per-thread stack */
                  (QEvent*)0);                                      /* no initialization event */

    QActive_start(AO_DsmApi,                                        /* global pointer to the active object */
                  ++prio,                                           /* priority */
                  l_dsmApiQueueSto, Q_DIM(l_dsmApiQueueSto),        /* event queue */
                  (void*)0, 0,                                      /* no per-thread stack */
                  (QEvent*)0);                                      /* no initialization event */

    QActive_start(AO_DS485,                                         /* global pointer to the active object */
                  ++prio,                                           /* priority */
                  l_ds485QueueSto, Q_DIM(l_ds485QueueSto),          /* event queue */
                  (void*)0, 0,                                      /* no per-thread stack */
                  (QEvent*)0);                                      /* no initialization event */

    QActive_start(AO_Receiver,                                      /* global pointer to the active object */
                  ++prio,                                           /* priority */
                  l_receiverQueueSto, Q_DIM(l_receiverQueueSto),    /* event queue */
                  (void*)0, 0,                                      /* no per-thread stack */
                  (QEvent*)0);                                      /* no initialization event */
}

void libqp_thread(void)
{
  /* run the QF application */
  QF_run();
}
